#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dyninput.h"

extern int INTERACTIVE;

static Source *current = NULL;

/* getch: getnext character from current input source */
int getch()
{
    int c;
    /* if source is stdin, just getchar() */
    if (!current) {
	return getchar();
    }
    /* check for pushed back character */
    else if (current->ungot) {
	c = current->ungot;
	current->ungot = 0;
	current->len--;
	return c;
    } else {
	if (--current->len < 0) {
	    current->len = 0;
	    return EOF;
	}
	else
	    return *(current->str++);
    }
}

/* ungetch: push character back onto current source */
int ungetch(char c)
{
    /* if source is stdin, just ungetc() */
    if (!current)
	return ungetc(c, stdin);
    /* if there is already a pushed back character, fail */
    if (current->ungot)
	return EOF;
    current->ungot = c;
    current->len++;
    return c;
}

/* inputString: set a string as current input source */
Source *inputString(char *str, int len)
{
    Source *bu = current;
    current = malloc(sizeof(Source));
    current->len = len;
    current->str  = str;
    current->ungot = 0;
    return bu;
}

/* restoreInput: restore a previous input source */
void restoreInput(Source *source)
{
    free(current);
    current = source;
}
