#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include "parse.h"
#include "token.h"
#include "variables.h"

void loadenv();

int INTERACTIVE = 0;

int main(int argc, char **argv)
{
    Stmts *stmts;
    int err;
    int fd;
    int i;
    char tmp[20], *s;
    
    /* command line arguments */
    extern char *program_invocation_name;
    if (argc > 1) {
	if ((fd = open(argv[1], O_RDONLY)) == -1) {
	    switch (errno) {
		case EACCES:
		    error(1, 0, "%s: permission denied!", argv[1]);
		    break;
		case ENOENT:
		    error(1, 0, "%s: file does not exist!", argv[1]);
		default:
		    break;
	    }
	    exit(1);
	} else {
	    close(0);
	    dup(fd);
	    close(fd);
	    program_invocation_name = argv[1];
	    /* get rid of "jojsh" as first arg */
	    argv++;
	    argc--;
	}
    }

    if (isatty(0)) {
	INTERACTIVE = 1;
    }
    
    // ignore terminal blocks if we're interactive
    if (INTERACTIVE) {
	signal(SIGTTOU, SIG_IGN);
	signal(SIGTTIN, SIG_IGN);
    }

    
    /* load environment and set default variables values */
    loadenv();
    //setvar("PS1", "jojsh> ");
    setvar("PS1", "[\\u@jojsh!! \\w]$ ");
    setvar("PS2", "> ");
    setvar("HISTIZE", "1000");

    /* load any special shell variables */
    sprintf(tmp, "%d", getpid());
    setvar("$", tmp);
    /* positional paramaters */
    for (i = 0; i < argc; i++) {
	sprintf(tmp, "%d", i);
	setvar(tmp, argv[i]);
    }

    while (1) {
	if (INTERACTIVE) {
	    fprintf(stderr, "%s", parsePrompt(getvar("PS1")));
	}
	err = 0;
	
	stmts = parseStmts(0, &err);
	if (! err) {
	    if (runStmts(stmts))
		break;
	} else {
	    char *str;
	    /*in case of error, chomp rest of input */
	    while (getToken(&str) != NL)
		free(str);
	}
	cleanStmts(stmts);
    }
    return 0;
}
