#ifndef PARSETREE_H
#define PARSETREE_H

typedef enum {
    ASGNMTS,
    CMDS,
    IFS,
    WHILES,
    FORS,
    EOFS
} StmtType;

typedef enum {
    STRINGP,
    DOUBLEP,
    VARP,
    BACKP
} PieceType;

/* "Base classes" and corresponding lists */

typedef struct {
    StmtType stmttype;
} Stmt;

typedef struct _Stmts {
    Stmt *stmt;
    struct _Stmts *next;
} Stmts;

typedef struct {
    PieceType piecetype;
} Piece;

typedef Piece **Word;

/* Redirect struct, for use with CmdStmt */

typedef struct {
    int from;
    Word to;
    enum {RD, WR, RDWR, APND} direction;
    enum {FALSE = 0, TRUE = 1} tofile;
} Red;

/* Stmt Nodes */

typedef struct {
    StmtType stmttype;
    char *var;
    Word val;
} AsgnmtStmt;

typedef struct _CmdStmt {
    StmtType stmttype;
    Word *args;
    struct _CmdStmt *next;
    enum {PIPEN, ANDN, ORN} nexttype;
    Red **reds;
} CmdStmt;

typedef struct {
    StmtType stmttype;
    Stmts *cond;
    Stmts *body;
    Stmts *elsebody;
} IfStmt;

typedef struct {
    StmtType stmttype;
    Stmts *cond;
    Stmts *body;
} WhileStmt;

typedef struct {
    StmtType stmttype;
    char *var;
    Word *vals;
    Stmts *body;
} ForStmt;

/* Piece nodes */

typedef struct {
    PieceType piecetype;
    char *string;
} StringPiece;

typedef struct _DoublePiece {
    PieceType piecetype;
    Piece **pieces;
} DoublePiece;

typedef struct {
    PieceType piecetype;
    char *var;
} VarPiece;

typedef struct {
    PieceType piecetype;
    Stmts *stmts;
} BackPiece;

/* "Public" functions (for use outside parse.c) */

Stmt *EOFStmt();
Stmts *parseStmts(int, int *);
int runStmts(Stmts *);

#endif
