#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <ctype.h>
const char *getvar(const char *);

#define HASHSIZE 101

typedef struct _Hashentry {
    char *name;
    char *data;
    struct _Hashentry *next;
} Hashentry;

static Hashentry vars[HASHSIZE];

/* hash: create a hash for a string and return it */
unsigned int hash(const char *s)
{
    static int init = 0;
    unsigned long long hashval;

    /* if this is the first time running hash, initialize hash table */
    if (init = 0) {
	int i;
	init = 1;
	for (i = 0; i < HASHSIZE; i++) {
	    vars[i].next = NULL;
	}
    }

    /* do the hashing */
    for (hashval = 0; *s != '\0'; s++)
	hashval = *s + 31 * hashval;
    return hashval % HASHSIZE;
}

/* setvar: set the value of a variable */
void setvar(const char *var, const char *val)
{
    Hashentry *loc;

    /* look up varibale name in hash table and set its value if it exists */
    for (loc = vars + hash(var); loc->next; loc = loc->next) {
	if (!strcmp(var, loc->name)) {
	    free(loc->data);
	    loc->data = strdup(val);
	    return;
	}
    }

    /* if the variable is new, allocate both the variable name and the value */
    loc->name = strdup(var);
    loc->data = strdup(val);
    loc->next = malloc(sizeof(Hashentry));
}

/* getvar: retrieve the value of a variable */
const char *getvar(const char *var)
{
    Hashentry *loc;
    
    /* look up variable name in hash table and get its value if it exists */
    for (loc = vars + hash(var); loc->next; loc = loc->next)
	if (!strcmp(var, loc->name))
	    return loc->data;
    /* if the variable doesn't exist, return an empty string */
    return "";
}

extern char **environ;

/* loadenv: load environment variables into hash table */
void loadenv()
{
    char **env = environ - 1;
    char *var, *val;
    while (*++env) {
	var = *env;
	val = *env;
	while (*++val != '=')
	    ;
	*val = '\0';
	setvar(var, val+1);
	*val = '=';
    }
}

#define CHUNKSIZE 20
#define MAXSIZE 100
char parsed[MAXSIZE + 1];
/* parseprompt: parse programmable prompt string */
const char *parsePrompt(const char *prompt) {
    const char *c;
    char *s;//, *parsed;
    int i = 0;//, size = CHUNKSIZE;

    for (c = prompt; *c; c++) {
	if (i >= MAXSIZE) {
	    parsed[MAXSIZE] = '\0';
	    return &parsed[0];
	}
	if (*c == '\\') {
	    switch (*++c) {
		case 'u': 
		    s = getlogin();
		    if (s) {
			strncpy(parsed + i, s, MAXSIZE - i);
			i += strlen(s);
		    }
		    break;
		case 'h':
		    gethostname(parsed + i, MAXSIZE - i);
		    i += strlen(parsed + i);
		    break;
		case 'W':
		    s = getcwd(NULL, 0);
		    if (!strcmp(s, getvar("HOME"))) {
			parsed[i++] = '~';
		    } else {
			strncpy(parsed + i, strrchr(s, '/')+1, MAXSIZE - i);
			i += strlen(strrchr(s, '/')+1);
		    }
		    free(s);
		    break;
		case 'w':
		    s = getcwd(NULL, 0);
		    if (!strncmp(s, getvar("HOME"), strlen(getvar("HOME")))) {
			parsed[i++] = '~';
			strncpy(parsed + i, s + strlen(getvar("HOME")), MAXSIZE - i);
			i += strlen(s + strlen(getvar("HOME")));
		    } else {
			strncpy(parsed + i, s, MAXSIZE - i);
			i += strlen(s);
		    }
		    free(s);
		    break;
		default:
		    c--;
		case '\\':
		    parsed[i++] = '\\';
		    break;
		    
	    }
	} else {
	    parsed[i++] = *c;
	}
    }
   parsed[i] ='\0';
   return &parsed[0];
}
#undef CHUNKSIZE
#undef MAXSIZE
